#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <sys/stat.h>

#include "libgsmcom.h"
#include "gsm_internal.h"
#include "gsm_sched.h"
#include "gsm_queue.h"



/*is call for application*/
 int  gsm_call_set_state(struct gsm_span *span, gsm_call_state_t callstate)
{
     int status = 0;
     int res = 0;
     
     
     if(!span)
     {
        gsm_log(GSM_LOG_WARNING,"gsm_call_set_state but  have no call\n");
        return 1;
     }

     
     span->calls.newstate = callstate;
     
     switch(callstate)
     {
          case GSM_CALL_STATE_DIALING://ring
          {
                //span->call_progress = 0;
                if(span->calls.direction == GCOM_DIRECTION_INCOMING)
                 {
                     break;
                 }    
                else
                {
                    char buf[512] = {0};
                    snprintf(buf, sizeof(buf), "%s%s;%s", "ATD", span->calls.data.called_num,GSM_CMD_END);
                    res = gsm_write_cmd(span, buf, strlen(buf));
                    span->call_progress = 1;
                }
          }
          break;
          case GSM_CALL_STATE_DIALED:
          {
                if(span->calls.direction == GCOM_DIRECTION_INCOMING)//recv clip
               {
                    status = GSM_CALL_STATE_DIALED;
                    struct gsm_call_data  event;
                    memset(&event, 0, sizeof(struct gsm_call_data));
                    //add num
                    memcpy(event.calling_num, span->calls.data.calling_num, sizeof(event.calling_num));
                    event.callid = span->calls.data.callid;//add other
                    event.sub = GSM_SUB_REAL;
                    gsm_call_ind( span->span_id, &event);
               }
          }
          break;
          case GSM_CALL_STATE_RINGING:
          {
                status = GSM_CALL_STATE_RINGING;

                gsm_call_report(span->span_id, span->calls.data.callid, status);
               
          }
          break;
          case GSM_CALL_STATE_ANSWERED://remote answer
          {

                if(span->calls.direction  == GCOM_DIRECTION_INCOMING)
                {
                     char buf[100] = {0};
                     snprintf(buf,sizeof(buf),"%s%s","ATA",GSM_CMD_END);
                     res = gsm_write_cmd(span, buf, strlen(buf));
                     gsm_call_set_state(span, GSM_CALL_STATE_UP);
                }
                else
                {
                    status = GSM_CALL_STATE_ANSWERED;
                    gsm_call_report(span->span_id, span->calls.data.callid, status);   
                    gsm_call_set_state(span, GSM_CALL_STATE_UP);
                    span->call_progress = 0;
                }
                         
          }
          break;
          case GSM_CALL_STATE_UP: 
          {

             span->heat = 0;
             break;
          }
          case GSM_CALL_STATE_TERMINATING://remote hangup
          {
                status = GSM_CALL_STATE_TERMINATING;
                gsm_log(GSM_LOG_WARNING,"GSM_CALL_STATE_TERMINATING spanid=%d\n",span->span_id);
                gsm_call_hangup_ind(span->span_id, 0, 1);
                
          }
          break;
          case GSM_CALL_STATE_TERMINATING_CMPL:
          {
                span->event_call_state = 0;
                span->alarm = 0;
                memset(&span->calls, 0, sizeof(span->calls));
              
          }
          break;
          case GSM_CALL_STATE_HANGUP:
          {//maybe sendsms in call
                 gsm_log(GSM_LOG_WARNING," GSM_CALL_STATE_HANGUP id=%d\n",span->calls.data.callid);
                 
                 char buf[100] = {0};
                 snprintf(buf,sizeof(buf),"%s%s","ATH",GSM_CMD_END);
                 res = gsm_write_cmd(span, buf, strlen(buf));
                 gsm_call_set_state(span, GSM_CALL_STATE_HANGUP_CMPL);
          }
          break;
          case GSM_CALL_STATE_HANGUP_CMPL:
          {

                status = GSM_CALL_STATE_HANGUP_CMPL;
                span->event_call_state = 0;
                span->alarm = 0;
                gsm_log(GSM_LOG_WARNING," GSM_CALL_STATE_HANGUP_CMPL id=%d\n",span->calls.data.callid);
                gsm_call_hangup_cfm(span->span_id, span->calls.data.callid,status);
                memset(&span->calls, 0, sizeof(span->calls));
          }
           break;
          default:
            break;
     }
     return 0;
}





int gsm_hangup_req(unsigned int span_id,  unsigned char callid)
{
     struct gsm_span *span = NULL;
    // char *hangup_cmd = NULL;
     if( (span_id <= 0)  || (span_id > GSM_MAX_SPANS))
        return 1;

    span = gsm_get_span(span_id);

     if(!span)
        return 1;

     //span state 

     if(span->span_state < GSM_SPAN_STATE_RUNNING)
         return 1;



      gsm_call_set_state(span, GSM_CALL_STATE_HANGUP);
      return 0;
     
}

int gsm_hangup_cfm(unsigned int span_id,  unsigned char callid)
{
     struct gsm_span *span = NULL;

     if( (span_id <= 0)  || (span_id > GSM_MAX_SPANS))
        return 1;

    span = gsm_get_span(span_id);

     if(!span)
        return 1;

     //span state 

     if(span->span_state < GSM_SPAN_STATE_RUNNING)
         return 1;

      
      gsm_call_set_state(span, GSM_CALL_STATE_TERMINATING_CMPL);
      return 0;
}


int gsm_call_cfm(unsigned int span_id,  unsigned char callid)
{
     struct gsm_span *span = NULL;
    // char *answer_cmd = NULL;

     if( (span_id <= 0)  || (span_id > GSM_MAX_SPANS))
        return 1;

    span = gsm_get_span(span_id);

     if(!span)
        return 1;

     //span state 

     if(span->span_state < GSM_SPAN_STATE_RUNNING)
         return 1;


      
      gsm_log(GSM_LOG_WARNING,"gsm_call_cfm and set state answered\n");
      gsm_call_set_state(span, GSM_CALL_STATE_ANSWERED);
      
      return 0;
}


//have no id response with sig_gsm_chan and libgsm span->calls
int gsm_call_req(unsigned int span_id,  struct gsm_call_data  *call_data)
{
    struct gsm_span *span = NULL ;

    int res = 0;
    
    if( (span_id <= 0)  || (span_id > GSM_MAX_SPANS))
        return 1;

    span = gsm_get_span(span_id);

     if(!span || !call_data)
        return 1;


    if(span->span_state < GSM_SPAN_STATE_RUNNING)
         return 1;
      

    memcpy(&span->calls.data, call_data, sizeof(span->calls.data));
    
    res = gsm_span_create_call(span, 0, GCOM_DIRECTION_OUTGOING);
    if(res)
        return res;
    
    gsm_call_set_state(span, GSM_CALL_STATE_DIALING);
    return 0;
}




int gsm_span_create_call(struct gsm_span *span, unsigned int call_id,int direct)
{
    unsigned int id = 0;
    //gsm_span_call_t *span_call = NULL;

    if(call_id)//not zero
    {
        if(span->calls.newstate > GSM_CALL_STATE_INIT)
        {
            gsm_log(GSM_LOG_WARNING, "gsm module spanno=%d is busy already have call\n",span->span_id);
            return 1;
        }
        id = call_id;
    }
    else
    {
        id = gsm_find_span_callid(span);
        if(id == 0)
        {
            gsm_log(GSM_LOG_WARNING,"span id=%d is in max calls now\n",span->span_id);
            return 1;
        }
    }

  
     span->calls.span = span;
     span->calls.data.callid = id;
     gsm_log(GSM_LOG_WARNING,"gsm_span_create_call id=%d\n",id);
     span->calls.data.direct = direct;
     
     span->calls.newstate = GSM_CALL_STATE_INIT;
     span->calls.direction = direct;

     return 0;
    
}

 int gsm_find_span_callid(struct gsm_span *span)
{
    int i = 0;
    //for(i=0;i<GSM_MAX_CALLS_IN_SPAN;i++)
    {
        if(span->calls.newstate == 0)
            return i+1;
    }

    return 0;
}

 int gsm_find_span_call_by_state(struct gsm_span* span, gsm_call_state_t state)
{
    int i = 0;
   // for(i=0;i<GSM_MAX_CALLS_IN_SPAN;i++)
    {
        {
            if(span->calls.newstate == state)
                return i+1;
        }
    }
    return 0;
}
